# sorry...
import os
os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "hide"

import pygame
import sys
import csv
from src.consts import *
from src.objects import *


def level(level_number, level_dir):
    if player_group.sprites():
        player = player_group.sprites()[0]
        player.health = PLAYER_MAX_HEALTH
        if player.dead_level:
            change = player.dead_level
            player.dead_level = 0
            return change
    else:
        player = None

    with open(f'./data/levels/level{level_number}.txt') as level_file:
        reader = csv.reader(level_file, escapechar='\\')
        for row in reader:
            if not row:
                continue
            if row[0] == 'wall':
                pos = int(row[1]), int(row[2])
                size = int(row[3]), int(row[4])
                Wall(pos, size)
            elif row[0] == 'npc':
                pos = int(row[1]), int(row[2])
                NPC(pos, row[3:])
            elif row[0] == 'gun':
                pos = int(row[1]), int(row[2])
                speed = 1/float(row[3])
                Gun(pos, speed)
            elif row[0] == 'next_level':
                pos = int(row[1]), int(row[2])
                size = int(row[3]), int(row[4])
                Next_Level(pos, size, 1)
            elif row[0] == 'prev_level':
                pos = int(row[1]), int(row[2])
                size = int(row[3]), int(row[4])
                Next_Level(pos, size, -1)

    camera = Camera(screen, player)

    clock = pygame.time.Clock()


    if level_dir >= 0:
        for level in level_group:
            if level.inc == -level_dir:
                new_pos = [
                    level.pos[0]+level.rect.width+1,
                    player.pos[1] + level.pos[1],
                ]
                # offset = player.pos[0] - new_pos[0], player.pos[1] - new_pos[1]
                offset = new_pos[0] - player.pos[0], new_pos[1] - player.pos[1]
                player.pos = new_pos
                start_pos = tuple(player.pos)
                break
    else:
        for level in level_group:
            if level.inc == -level_dir:
                new_pos = [
                    level.pos[0]-player.rect.width-1,
                    player.pos[1] + level.pos[1],
                ]
                # offset = player.pos[0] - new_pos[0], player.pos[1] - new_pos[1]
                offset = new_pos[0] - player.pos[0], new_pos[1] - player.pos[1]
                player.pos = new_pos
                start_pos = tuple(player.pos)
                break

    for b in background_draw_group:
        b.offset(offset)

    # game loop
    while True:
        # print(value)
        keys = pygame.key.get_pressed()

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()


        for s in key_group.sprites():
            s.keys(keys)

        # update the player
        player.move(collidable_group)
        player.take_damage(damage_group)
        if player.health == 0:
            player.dead_level = -1
            return 1
        camera.update(player)

        # update targets of enemies
        for s in interact_group.sprites():
            s.target(player)

        for group in move_group:
            group.update(camera.pos)

        # draw groups
        for group in draw_groups:
            group.draw(screen)

        # draw changes
        pygame.display.update()
        clock.tick(FPS)

        # check for next/previous level
        for level in level_group:
            if level.change:
                player.pos[1] -= level.pos[1]
                background.offset((0, -level.pos[1]))
                return level.inc
        # temp=True
        # while temp:
        #     for event in pygame.event.get():
        #         if event.type == pygame.KEYDOWN:
        #             temp=False
        #         if event.type == pygame.QUIT:
        #             sys.exit()



if __name__ == '__main__':
    # init pygame
    pygame.init()

    try:
        pygame.mixer.music.load(r'.\data\music\Kmouse - dreamscape.mp3')
        pygame.mixer.music.play()
    except Exception:
        print('could not load music')

    # init the screen
    screen = pygame.display.set_mode(SCREEN_SIZE)

    # create groups
    player_group = pygame.sprite.Group()
    level_group = pygame.sprite.Group()

    move_group = pygame.sprite.Group()

    player_draw_group = pygame.sprite.Group()
    collidable_draw_group = pygame.sprite.Group()
    background_draw_group = pygame.sprite.Group()
    object_draw_group = pygame.sprite.Group()
    message_draw_group = pygame.sprite.Group()

    collidable_group = pygame.sprite.Group()
    interact_group = pygame.sprite.Group()
    key_group = pygame.sprite.Group()
    damage_group = pygame.sprite.Group()

    transient_group = pygame.sprite.Group()

    draw_groups = [
        background_draw_group,
        collidable_draw_group,
        object_draw_group,
        player_draw_group,
        message_draw_group,
    ]

    Player.containers     = player_draw_group, player_group, move_group, key_group
    Background.containers = background_draw_group, move_group
    Healthbar.containers  = message_draw_group, interact_group

    Message.containers    = transient_group, message_draw_group, key_group
    Wall.containers       = transient_group, collidable_draw_group, move_group, collidable_group
    Bullet.containers     = transient_group, object_draw_group, move_group, interact_group, damage_group
    Gun.containers        = transient_group, object_draw_group, move_group, interact_group
    NPC.containers        = transient_group, object_draw_group, move_group, interact_group, key_group
    Next_Level.containers = transient_group, move_group, interact_group, level_group

    player = Player((0, 90))

    # initialise the background
    background = Background('temp.png', screen)

    # initialise the Healthbar
    Healthbar()

    i = 1
    level_dir = 1
    while i < 10:
        level_dir = level(i, level_dir)
        i += level_dir
        for s in transient_group.sprites():
            s.kill()
        # print(transient_group.sprites())
