import pygame
from random import randrange, random
import os
from .consts import *
from .misc import *
from math import ceil
import math

__all__ = ['Gun', 'Bullet']

class Gun(pygame.sprite.Sprite):
    def __init__(self, pos, speed, background_pos=(0, 0)):
        pygame.sprite.Sprite.__init__(self, Gun.containers)

        # self.base_image = pygame.image.load('data/images/gun.jpg')
        self.base_image = pygame.Surface(GUN_SIZE)
        self.base_image.fill((100, 100, 100))
        self.image = self.base_image.copy()

        self.rect = self.image.get_rect()

        self.pos = list(pos)
        self.update_pos(background_pos)

        self.firing_rate = ceil(speed*FPS)

        self.player_dist = 10000

    @property
    def center(self):
        return self.pos[0] + self.rect.width/2, self.pos[1] + self.rect.height/2

    def update_pos(self, background_pos):
        self.rect.topleft = (
            self.pos[0] + background_pos[0],
            self.pos[1] + background_pos[1],
        )

    def target(self, player):
        self.image = self.base_image.copy()
        player_rel_pos = player.rect.centerx - self.rect.left, player.rect.centery - self.rect.top
        p1, p2 = short_line((self.rect.width/2, self.rect.height/2), player_rel_pos, GUN_LENGTH)

        self.player_dist = distance_to(self.rect.center, player.rect.center)
        pygame.draw.line(self.image, BLACK, p1, p2, 5)

    def update(self, background_pos):
        self.update_pos(background_pos)
        if randrange(self.firing_rate) == 0 and self.player_dist < GUN_SEEN:
            if self.player_dist < GUN_TARGET:
                shake = GUN_SHAKE_TARGETED
            else:
                shake = GUN_SHAKE
            Bullet(self.center, shake, background_pos)


class Bullet(pygame.sprite.Sprite):
    def __init__(self, pos, shake, background_pos=(0, 0)):
        pygame.sprite.Sprite.__init__(self, Bullet.containers)

        self.base_image = pygame.image.load('data/images/bullet.png')
        self.image = self.base_image.copy()

        self.rect = self.image.get_rect()

        self.pos = list(pos)
        self.pos[0] -= self.rect.width/2
        self.pos[1] -= self.rect.height/2
        self.update_pos(background_pos)

        self.damage = BULLET_DAMAGE

        self.shake = shake
        self.dir = None
        self.age = 0

    def update_pos(self, background_pos):
        self.rect.topleft = (
            self.pos[0] + background_pos[0],
            self.pos[1] + background_pos[1],
        )

    def target(self, player):
        if pygame.sprite.spritecollideany(self, [player], False):
            self.kill()
        if self.dir is None:
            # Get the total length of the line
            dir1 = (self.pos[0] + self.rect.width/2) - (player.pos[0] + player.rect.width/2)
            dir2 = (self.pos[1] + self.rect.height/2) - (player.pos[1] + player.rect.height/2)
            rads = math.atan2(-dir2, -dir1) + random()*self.shake*2-self.shake
            self.dir = math.cos(rads) * BULLET_VEL, math.sin(rads) * BULLET_VEL

    def update(self, background_pos):
        self.update_pos(background_pos)
        if self.dir is not None:
            self.pos[0] += self.dir[0]
            self.pos[1] += self.dir[1]
        if self.age > BULLET_MAX_AGE:
            self.kill()
        self.age += 1
